@@ -11,15 +11,42 @@
 TortoiseHg About dialog - PyQt4 version
 """
 
+from __future__ import absolute_import
+
 import sys
 
-from tortoisehg.hgqt import qtlib
-from tortoisehg.util import version, hglib, paths
-from tortoisehg.util.i18n import _
+from .qtcore import (
+    PYQT_VERSION_STR,
+    QSettings,
+    QSize,
+    QT_VERSION_STR,
+    QTimer,
+    QUrl,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QLabel,
+    QLayout,
+    QPixmap,
+    QPlainTextEdit,
+    QVBoxLayout,
+)
+from .qtnetwork import (
+    QNetworkAccessManager,
+    QNetworkRequest,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.QtNetwork import QNetworkAccessManager, QNetworkRequest
+from ..util import (
+    hglib,
+    paths,
+    version,
+)
+from ..util.i18n import _
+from . import qtlib
 
 class AboutDialog(QDialog):
     """Dialog for showing info about TortoiseHg"""
@@ -52,7 +79,7 @@ class AboutDialog(QDialog):
         self.copyright_lbl = QLabel()
         self.copyright_lbl.setAlignment(Qt.AlignCenter)
         self.copyright_lbl.setText('\n'
-                + _('Copyright 2008-2016 Steve Borho and others'))
+                + _('Copyright 2008-2017 Steve Borho and others'))
         self.vbox.addWidget(self.copyright_lbl)
         self.courtesy_lbl = QLabel()
         self.courtesy_lbl.setAlignment(Qt.AlignCenter)
@@ -66,7 +93,8 @@ class AboutDialog(QDialog):
         self.download_url_lbl.setTextInteractionFlags(Qt.LinksAccessibleByMouse)
         self.download_url_lbl.setOpenExternalLinks(True)
         self.download_url_lbl.setText('<a href=%s>%s</a>' %
-                ('http://tortoisehg.org', _('You can visit our site here')))
+                                      ('https://tortoisehg.bitbucket.io',
+                                       _('You can visit our site here')))
         self.vbox.addWidget(self.download_url_lbl)
 
         # Let's have some space between the url and the buttons.
@@ -109,7 +137,7 @@ class AboutDialog(QDialog):
 
     @pyqtSlot()
     def getUpdateInfo(self):
-        verurl = 'http://tortoisehg.bitbucket.org/curversion.txt'
+        verurl = 'https://tortoisehg.bitbucket.io/curversion.txt'
         # If we use QNetworkAcessManager elsewhere, it should be shared
         # through the application.
         self._netmanager = QNetworkAccessManager(self)
@@ -165,7 +193,7 @@ class AboutDialog(QDialog):
 
     def _readsettings(self):
         s = QSettings()
-        self.restoreGeometry(s.value('about/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'about/geom'))
 
     def _writesettings(self):
         s = QSettings()
@@ -210,7 +238,7 @@ class LicenseDialog(QDialog):
 
     def _readsettings(self):
         s = QSettings()
-        self.restoreGeometry(s.value('license/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'license/geom'))
 
     def _writesettings(self):
         s = QSettings()
