@@ -8,19 +8,50 @@
 Main Qt4 application for TortoiseHg
 """
 
+from __future__ import absolute_import
+
 import os
 import subprocess
 import sys
-from tortoisehg.util import paths, hglib
-from tortoisehg.util.i18n import _
 
-from tortoisehg.hgqt import cmdcore, cmdui, qtlib, mq, repotab, serve
-from tortoisehg.hgqt.reporegistry import RepoRegistryView
-from tortoisehg.hgqt.docklog import LogDockWidget
-from tortoisehg.hgqt.settings import SettingsDialog
+from .qtcore import (
+    QSettings,
+    QT_VERSION,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAction,
+    QActionGroup,
+    QApplication,
+    QComboBox,
+    QFileDialog,
+    QKeySequence,
+    QMainWindow,
+    QMenu,
+    QMenuBar,
+    QShortcut,
+    QSizePolicy,
+    QToolBar,
+    qApp,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util import (
+    hglib,
+    paths,
+)
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    mq,
+    qtlib,
+    repotab,
+    serve,
+)
+from .docklog import LogDockWidget
+from .reporegistry import RepoRegistryView
+from .settings import SettingsDialog
 
 class Workbench(QMainWindow):
     """hg repository viewer/browser application"""
@@ -78,9 +109,18 @@ class Workbench(QMainWindow):
                                     self.cloneRepository)
             self.dockMenu.addAction(_('&Open Repository...'),
                                     self.openRepository)
-            qt_mac_set_dock_menu(self.dockMenu)
-            # On Mac OS X, we do not want icons on menus
-            qt_mac_set_menubar_icons(False)
+            if QT_VERSION < 0x50000:
+                from .qtgui import (
+                    qt_mac_set_dock_menu,
+                    qt_mac_set_menubar_icons,
+                )
+                qt_mac_set_dock_menu(self.dockMenu)
+                # On Mac OS X, we do not want icons on menus
+                qt_mac_set_menubar_icons(False)
+            else:
+                self.dockMenu.setAsDockMenu()
+                qapp = QApplication.instance()
+                qapp.setAttribute(Qt.AA_DontShowIconsInMenus, True)
 
         self._dialogs = qtlib.DialogKeeper(
             lambda self, dlgmeth: dlgmeth(self), parent=self)
@@ -102,6 +142,7 @@ class Workbench(QMainWindow):
         tw.currentTitleChanged.connect(self._updateWindowTitle)
         tw.historyChanged.connect(self._updateHistoryActions)
         tw.makeLogVisible.connect(self._setConsoleVisible)
+        tw.taskTabVisibilityChanged.connect(self._updateTaskTabVisibilityAction)
         tw.toolbarVisibilityChanged.connect(self._updateToolBarActions)
 
         self.setCentralWidget(tw)
@@ -154,6 +195,8 @@ class Workbench(QMainWindow):
         newaction(_("&Close Repository"), self.closeCurrentRepoTab,
                   shortcut='Close', enabled='repoopen', menu='file')
         newseparator(menu='file')
+        self.menuFile.addActions(self.repoTabsWidget.tabSwitchActions())
+        newseparator(menu='file')
         newaction(_('&Settings'), self.editSettings, icon='thg-userconfig',
                   shortcut='Preferences', menu='file')
         newseparator(menu='file')
@@ -201,6 +244,11 @@ class Workbench(QMainWindow):
                   menu='view')
         newseparator(menu='view')
 
+        a = newaction(_('Show Tas&k Tab'), shortcut='Alt+0', checkable=True,
+                      enabled='repoopen', menu='view')
+        a.triggered.connect(self._setRepoTaskTabVisible)
+        self.actionTaskTabVisible = a
+
         self.actionGroupTaskView = QActionGroup(self)
         self.actionGroupTaskView.triggered.connect(self._onSwitchRepoTaskTab)
         def addtaskview(icon, label, name):
@@ -455,7 +503,7 @@ class Workbench(QMainWindow):
         if urlindex < 0:
             return
         opindex = {'incoming': 0, 'pull': 0, 'outgoing': 1, 'push': 1}[op]
-        return self.urlCombo.itemData(urlindex).toPyObject()[opindex]
+        return self.urlCombo.itemData(urlindex)[opindex]
 
     @pyqtSlot(int)
     def _updateSyncUrl(self, index):
@@ -476,7 +524,7 @@ class Workbench(QMainWindow):
             tooltip = _('There are no configured sync paths.\n'
                         'Open the Synchronize tab to configure them.')
         else:
-            tooltip = self.urlCombo.itemData(index, Qt.ToolTipRole).toString()
+            tooltip = self.urlCombo.itemData(index, Qt.ToolTipRole)
         self.urlCombo.setToolTip(tooltip)
 
     def _updateSyncActionToolTip(self, index):
@@ -488,7 +536,7 @@ class Workbench(QMainWindow):
                 'push':     _('Push outgoing changes'),
                 }
         else:
-            pullurl, pushurl = self.urlCombo.itemData(index).toPyObject()
+            pullurl, pushurl = self.urlCombo.itemData(index)
             tooltips = {
                 'incoming': _('Check for incoming changes from\n%s') % pullurl,
                 'pull':     _('Pull incoming changes from\n%s') % pullurl,
@@ -497,7 +545,7 @@ class Workbench(QMainWindow):
                 }
 
         for a in self.synctbar.actions():
-            op = str(a.data().toString())
+            op = str(a.data())
             if op in tooltips:
                 a.setToolTip(tooltips[op])
 
@@ -605,7 +653,14 @@ class Workbench(QMainWindow):
     def _onSwitchRepoTaskTab(self, action):
         rw = self._currentRepoWidget()
         if rw:
-            rw.switchToNamedTaskTab(str(action.data().toString()))
+            rw.switchToNamedTaskTab(str(action.data()))
+
+    @pyqtSlot(bool)
+    def _setRepoTaskTabVisible(self, visible):
+        rw = self._currentRepoWidget()
+        if not rw:
+            return
+        rw.setTaskTabVisible(visible)
 
     @pyqtSlot(bool)
     def _setConsoleVisible(self, visible):
@@ -706,6 +761,7 @@ class Workbench(QMainWindow):
 
         # Update actions affected by repo open/close/change
         self._updateTaskViewMenu()
+        self._updateTaskTabVisibilityAction()
         self._updateToolBarActions()
 
     @pyqtSlot()
@@ -741,7 +797,7 @@ class Workbench(QMainWindow):
                 self.actionSelectTaskPbranch.setVisible('pbranch' in exts)
             name = repoWidget.currentTaskTabName()
             for action in self.actionGroupTaskView.actions():
-                action.setChecked(str(action.data().toString()) == name)
+                action.setChecked(str(action.data()) == name)
             self.lockToolAction.setVisible('simplelock' in exts)
         self._updateShowConsoleAction()
 
@@ -749,6 +805,11 @@ class Workbench(QMainWindow):
                               if a.isVisible()):
             a.setShortcut('Alt+%d' % (i + 1))
 
+    @pyqtSlot()
+    def _updateTaskTabVisibilityAction(self):
+        rw = self._currentRepoWidget()
+        self.actionTaskTabVisible.setChecked(bool(rw) and rw.isTaskTabVisible())
+
     @pyqtSlot()
     def _updateHistoryActions(self):
         'Update back / forward actions'
@@ -820,7 +881,7 @@ class Workbench(QMainWindow):
     def _runSyncAction(self, action):
         w = self._currentRepoWidget()
         if w:
-            op = str(action.data().toString())
+            op = str(action.data())
             w.setSyncUrl(self._syncUrlFor(op) or '')
             getattr(w, op)()
 
@@ -872,7 +933,9 @@ class Workbench(QMainWindow):
                                      _("Goto revision"),
                                      _("Enter revision identifier"))
         if ok:
-            self.gotorev(rev)
+            w = self._currentRepoWidget()
+            assert w
+            w.gotoRev(rev)
 
     @pyqtSlot(str)
     def gotorev(self, rev):
@@ -1047,33 +1110,33 @@ class Workbench(QMainWindow):
     def restoreSettings(self):
         s = QSettings()
         wb = "Workbench/"
-        self.restoreGeometry(s.value(wb + 'geometry').toByteArray())
-        self.restoreState(s.value(wb + 'windowState').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, wb + 'geometry'))
+        self.restoreState(qtlib.readByteArray(s, wb + 'windowState'))
         self._actionDockedConsole.setChecked(
-            s.value(wb + 'dockedConsole', True).toBool())
+            qtlib.readBool(s, wb + 'dockedConsole', True))
 
         lastreposyncpaths = {}
         npaths = s.beginReadArray('lastreposyncpaths')
         for n in range(npaths):
             s.setArrayIndex(n)
-            root = unicode(s.value('root').toString())
-            lastreposyncpaths[root] = s.value('path').toString()
+            root = qtlib.readString(s, 'root')
+            lastreposyncpaths[root] = qtlib.readString(s, 'path')
         s.endArray()
         self._lastRepoSyncPath = lastreposyncpaths
 
-        save = s.value(wb + 'saveRepos').toBool()
+        save = qtlib.readBool(s, wb + 'saveRepos')
         self.actionSaveRepos.setChecked(save)
-        savelastsyncpaths = s.value(wb + 'saveLastSyncPaths').toBool()
+        savelastsyncpaths = qtlib.readBool(s, wb + 'saveLastSyncPaths')
         self.actionSaveLastSyncPaths.setChecked(savelastsyncpaths)
 
-        openreposvalue = unicode(s.value(wb + 'openrepos').toString())
+        openreposvalue = qtlib.readString(s, wb + 'openrepos')
         if openreposvalue:
             openrepos = openreposvalue.split(',')
         else:
             openrepos = []
         # Note that if a "root" has been passed to the "thg" command,
         # "lastactiverepo" will have no effect
-        lastactiverepo = unicode(s.value(wb + 'lastactiverepo').toString())
+        lastactiverepo = qtlib.readString(s, wb + 'lastactiverepo')
         self.repoTabsWidget.restoreRepos(openrepos, lastactiverepo)
 
         # Clear the lastactiverepo and the openrepos list once the workbench state
